/*
EEPROM 93AA86C soll am Mega162 funktionieren.
Hier soll das SPI-Interface benutzt werden, um das EEPROM anzusteuern.
Wichtig: /SS-Pin mu als Ausgang definiert sein, sonst funktioniert der Master-mode nicht.
Nach dem Schreibbefehl mu tatschlich diese Abfrage auf Ready/busy erfolgen.


weiter siehe eeprom_1/SPI_EEPROM_Write_Read_Alphabet_8.c

EPROM			am Mega162

CS, Pin1		PC0, Pin21			<<<<<-----<<<<--- 
CLK, Pin2		PB7, Pin8
Din, Pin3		MOSI, PB5, Pin6
Dout, Pin4		MISO, PB6, Pin7

Pin5 = GND
ORG immer an GND
PE immer an +
Pin8 +5V

Table 1 - 4 		8-bit modus:


Befehl	Startbit	OPcode	Adresse					data_in		data_out	bentigte Zyklenanzahl

READ		1			10	A10...A0							D7..D0
EWEN		1			00	1	1	XXXXX XXXX					high-Z
ERASE		1			11	A10...A0							RDY/BSY
ERAL		1			00	1	0	XXXXX XXXX					RDY/BSY
WRITE		1			01	A10...A0				D7..D0		RDY/BSY
WRAL		1			00	0	1	XXXXX XXXX		D7..D0		RDY/BSY
EWDS		1			00	0	0	XXXXX XXXX					high-Z


Die positive Flanke an CLK dient, um ein Bit einzulesen oder auszugeben.

__________________________________________
Datasheet eines beliebigen AVR mit SPI:


SPI Control Register  SPCR		Bit 7	 6 		5 		4 		3 		2 		1 		0		
									SPIE SPE 	DORD 	MSTR 	CPOL 	CPHA 	SPR1 	SPR0

 Bit 5  DORD: Data Order
When the DORD bit is written to one, the LSB of the data word is transmitted first.
When the DORD bit is written to zero, the MSB of the data word is transmitted first.
__________________________________________


Nach Anlegen der Spannung befindet es sich im EWDS (erase / write disabled ) Zustand.
Zum Schreiben oder Lschen mu zuvor EWEN (erase / write ebnable) gegeben worden sein.
Der Schreib-Modus bleibt aktiv, bis wieder ein EWDS gegeben wird.
Read ist unabhngig von EWDS oder EWEN. Hlt man CS high, knnen mehrere Bytes nacheinander gelesen werden.
Read gibt ein Dummy-Null-Bit aus, direkt nachdem der Read-Befehl gegeben wurde.

Im Fall von erase oder write dient DO der Statusabfrage, wenn CS gleich wieder auf high gesetzt wurde. 
DO wird auf high abgefragt. Falls high -->> device ready
Falls CS die ganze write- oder erase-Zeit auf low gehalten wird, ist DO hochimpedant. Erst danach wird DO zu high.

Version2:
Habe das Datenblatt falsch verstanden. In der Tabelle ist mit dem "Start-Bit" gleichzeitig die "start-condition" gemeint, 
die ich als separates Unterprogramm angelegt habe. Deshalb alle Unterprogramme so ndern, da sie direkt das Start-Bit 
enthalten (das habe sie bereits) und den Teil mit der Start-condition weg lassen.

Version3:
Fehler:

Adresse wird vom MSB her seriell ausgegeben. Das ist falsch. Man mte oben die unntigen Bits ausmaskieren 
und vom ersten relevanten bit aus loslegen mit der seriellen Ausgabe. Also X << 5 vorher geben.

Version4:
nochmals CS auf low ziehen, um das Ready busy auszuwerten.
Es wird irgendwann 200 gelesen. Bei Adresse 31 oder 32.
________________________
93AA86C_SPI_1.c
Version1:
Geschrieben wird nur dann, wenn 4ms gewartet wird nach dem bertragen von Adresse und Daten an das EEPROM.
Schreiben und Lesen aufs EEPROM funktionieren jetzt, jedoch wird immer nur der Halbe Wert angezeigt.
Vermutlich wird beim Lesebefehl eine L-H-Flanke des Taktes unterschlagen, weil das EEPROM noch Zeit braucht, 
um nach der Angabe der Adresse sofort die passenden Bytes auszugeben. Es Fehlt die Ausgabe des MSB der Daten.
Mittels Oszilloskop schauen, ob tatschlich das gelesene Byte die origonale Zahl an Pin DO anliegt, 
oder ob da eine Taktflanke verloren gegangen ist.




*/
#include <avr/io.h>
//#define F_CPU 8000000
#include <util/delay.h>
#include "LCD-Routines.h"		//fr LCD-Ausgabe
#include <util/delay.h>

#define _93AA86C_CS_PORT	 	PORTC
#define _93AA86C_CS_DDR 		DDRC
#define _93AA86C_PORT		 	PORTB
#define _93AA86C_DDR 			DDRB
#define _93AA86C_PIN 			PINB

#define _93AA86C_CS		(1 << 0)	//L --> deselected, standby mode	 H -->> device selected
#define _93AA86C_CLK	(1 << 7)	//clock, L-H-Flanke ist relevant
#define _93AA86C_DI		(1 << 5)	//data in
#define _93AA86C_DO		(1 << 6)	//data out		ready / busy 


#define CS_h			_93AA86C_CS_PORT |= _93AA86C_CS;
#define CS_l			_93AA86C_CS_PORT &= ~_93AA86C_CS;
#define CLK_h			_93AA86C_PORT |= _93AA86C_CLK;
#define CLK_l			_93AA86C_PORT &= ~_93AA86C_CLK;
#define DI_h			_93AA86C_PORT |= _93AA86C_DI;
#define DI_l			_93AA86C_PORT &= ~_93AA86C_DI;
#define DO_h			_93AA86C_PORT |= _93AA86C_DO;		//wird nur als Eingang genutzt
#define DO_l			_93AA86C_PORT &= ~_93AA86C_DO;		//wird nur als Eingang genutzt

#define EWEN_high		0b00100110
#define EWEN_low		0b00000000		//dont care

#define EWDS_high		0b00100000
#define EWDS_low		0b00000000		//dont care

#define ERAL_high		0b00100100
#define ERAL_low		0b00000000		//dont care

#define READ_low		0b00000000		//adresse & 0xFF
#define READ_high		0b00110000		//Achtung! nur die unteren 3 Bits drfen zu Veroderung heran gezogen werden! ~0xF8
//#define READ_high		0b00110A11, A10, A9			0b00110000 | (adresse >> 8)

#define ERASE_high		0b00111000		//			0b00111000 | (adresse >> 8)
#define ERASE_low		0b00000000		//adresse & 0xFF

#define WRITE_high		0b00101000		//			0b00101000 | (adresse >> 8)
//#define WRITE_low		0b00000000		//adresse & 0xFF

#define WRAL_high		0b00100010		//			0b00100010 | (adresse >> 8)
#define WRAL_low		0b00000000		//adresse & 0xFF

char sss[10]; 
uint16_t	adressenzaehler = 0;		//gibt die EEPROM-Adresse an

extern void ItoA_klein( int z, char* Buffer, uint8_t Stellenanzahl );		//fr LCD-Ausgabe
void waithalfcycle (void);
//void init_93AA86C	(void);
void EWEN_93AA86C	(void);
void EWDS_93AA86C	(void);
void WRITE_93AA86C	(uint16_t adresse, uint8_t daten);
void WRAL_93AA86C	(uint8_t daten);
uint16_t READ_93AA86C	(uint16_t adresse);
//uint8_t READ_93AA86C	(uint16_t adresse);
void ERASE_93AA86C	(uint16_t adresse);
void ERAL_93AA86C	(void);
void SPI_MasterInit(void);
//unsigned char SPI_MasterTransmit(unsigned char sendData);
uint8_t SPI_MasterTransmit(unsigned char sendData);

void SPI_MasterInit(void)
{
//Wichtig! Wichtig! Wichtig! Wichtig! Wichtig! 
//SS-Pin mu als Ausgang konfiguriert sein, sonst schaltet sich der Master-modus wieder sofort ab.

_93AA86C_CS_DDR |= (1 << 0);				//PORTC, PC0 als Ausgang schalten, LSB auf 1 stellen
_93AA86C_CS_PORT &= ~(1 << 0);				//PORTC, PC0 auf 0 stellen; L --> deselected, standby mode


_93AA86C_PORT &= ~(1 << 5) | ~(1 << 7) ;	//PB5 und PB7 alle Ausgnge auf low
//_93AA86C_PORT &= 0x00 ;	//alle Ausgnge auf low
_93AA86C_DDR |= (1 << 4) |(1 << 5) | (1 << 7);//PB5 und PB7 Ausgang fr CLK, DI des EEPROMs, PB4 = SS mu Ausgang sein, sonst Fehlfunktion

//PORTB = _BV(PB5) | _BV(PB2);        // SCK and PB2 high
//CS_Pin_PORT |= ( 1 << CS_Pin_SPI_EEPROM );		// CS-Pin auf high setzen

/* Enable SPI, Master, set clock rate fck/16 */
SPCR = (1<<SPE)|(1<<MSTR)|(1<<SPR1)|(1<<SPR0);	//SPI einschalten, master SPI mode ,fclock/128 bei Mega48 und Mega8, Mega162
//SPCR = (1<<SPE)|(1<<MSTR);						//SPI einschalten, master SPI mode ,fclock/4 bei Mega48 
//SPCR = (1<<SPE)|(1<<MSTR)|(1<<SPR0);		//SPI einschalten, master SPI mode ,fclock/16 bei Mega48 und Mega8
//SPCR = (1<<SPE)|(1<<MSTR)|(1<<SPR1);		//SPI einschalten, master SPI mode ,fclock/64 bei Mega48 und Mega8, Mega162

CS_l;
}


//unsigned char SPI_MasterTransmit(unsigned char sendData)
uint8_t SPI_MasterTransmit(unsigned char sendData)
{
  /* Start transmission */
  SPDR = sendData;

  /* Wait for transmission complete */
  while(!(SPSR & (1<<SPIF))) {}		//bleibt hier hngen!, wenn SS-Pin = PB4 als Eingang definiert ist. Mu Ausgang sein!!!!!!!!!!!!
/*  while(!(SPSR & (1<<SPIF))) 
		{
		uint8_t k;
		for (k = 0; k <= 100; k++)		//luft komplett durch, Programm bleibt hier hngen
			{
			_delay_ms (50);
			lcd_setcursor( 10, 4 );		//Stelle / Zeile
			ItoA_klein( k, sss, 5);		//selbst geschriebenes ItoA mit Leerzeichen vorne dran, schreibt rechtsbndig mit fhrenden Leerzeichen!
			lcd_string( sss );
			}
		}		//
*/
  return SPDR;
}


void EWEN_93AA86C	(void)		//Erase/Write Enable (EWEN)			14 clock-cycles
{
CS_h;

SPI_MasterTransmit(EWEN_high);
SPI_MasterTransmit(EWEN_low);

CS_l;
}


void EWDS_93AA86C	(void)		//Erase/Write Enable (EWEN)			14 clock-cycles
{
CS_h;

SPI_MasterTransmit(EWDS_high);
SPI_MasterTransmit(EWDS_low);

CS_l;
}

/*
es scheint etwas geschrieben zu werden, da auch etwas gelesen werden kann!!!
komischerweise geht der Ausgang des EEPROMs erst dann von high auf low, 
wenn das Startbit des Lesebefehls kommt und nicht schon am Ende des write-Befehls.
Aber so wird wenigstens etwas ausgegeben an DO. Nur liest das der Mikrocontroller nicht.
*/


void WRITE_93AA86C	(uint16_t adresse, uint8_t daten)		//22 clock-cycles, danach 6 ms warten, falls man kein Polling macht
{
CS_h;

SPI_MasterTransmit(WRITE_high  | ((adresse >> 8) & 0x07));//MSB der 16-Bit-Adresse und davon nur die 3 untersten Bits
SPI_MasterTransmit(adresse & 0xFF);
//CS_l;		//luft auch!
SPI_MasterTransmit(daten);

CS_l;


DO_l;			//PORTB, PB6 , den Eingang auf low setzen
_delay_ms(4);	//es wird nur dan geschrieben, wenn hier die Wartezeit gro genug ist!

CS_h;			//ab hier irgendwie 3....6 ms verbraten, um die Schreibzeit abzuwarten
//SPI_MasterTransmit(0x55);		//irgendeine zahl geben, damit DO auf 0 geht, um busy anzuzeigen.
SPI_MasterTransmit(0x10);		//irgendeine zahl geben, damit DO auf 0 geht, um busy anzuzeigen.
					//3ms .... 12 ms warten
//while(~(_93AA86C_PIN & (1<<_93AA86C_DO))) {}	//Abfrage, ob Eingangs-Pin 0 geworden ist -->> Programm bleibt hngen!!!
//while(!(_93AA86C_PIN & (1<<_93AA86C_DO))) {}	//Abfrage, ob Eingangs-Pin 0 geworden ist -->> Programm bleibt hngen!!!
while(_93AA86C_PIN & (1<<_93AA86C_DO)) {}	//Abfrage, ob Eingangs-Pin 1 geworden ist. Prft, ob dieses eine Bit 1 ist.
_delay_us(20);
CS_l;


_delay_us(20);
CS_h;				//kann entfallen
_delay_us(20);
CS_l;				//kann entfallen

}


void WRAL_93AA86C	(uint8_t daten)		//22 clock-cycles
{
CS_h;

SPI_MasterTransmit(WRAL_high);
SPI_MasterTransmit(WRAL_low);
SPI_MasterTransmit(daten);

CS_l;

}


 //als uint8_t erscheinen hier negative Zahlen
uint16_t READ_93AA86C	(uint16_t adresse)		//22 clock-cycles
//uint8_t READ_93AA86C	(uint16_t adresse)		//22 clock-cycles
{
uint8_t daten;
CS_h;	
//auf OSZI ist 110 am Anfang nicht zu erkennen. Oft ist es Dauer-high! Abhilfe:
//die oberen vier Bits von "adresse" mssen ausmaskiert werden, da sie sonst bei der Veroderung 
//die Befehls-Bits auf high stellen knnen, falls eine grere Adresse als erlaubt gegeben wird.
SPI_MasterTransmit(READ_high  | ((adresse >> 8) & 0x07));	//MSB der 16-Bit-Adresse und davon nur die 3 untersten Bits
SPI_MasterTransmit(adresse & 0xFF);							//LSB der 16-Bit-Adresse


//SPI_MasterTransmit(0x00);		//dummy read
//SPI_MasterTransmit(0x00);		//dummy read
daten = SPI_MasterTransmit(0x00);				//daten lesen, wichtig! daten = spimastertransmit (); das gelesene noch an die Variable zuweisen.
//liest die Hlfte der ZAHL
CS_l;
//return (0xFF);	//auf Display erscheint 255
return (daten);
}


void ERASE_93AA86C	(uint16_t adresse)		//14 clock-cycles
{
CS_h;	

SPI_MasterTransmit(ERASE_high  | ((adresse >> 8) & 0x07));//MSB der 16-Bit-Adresse und davon nur die 3 untersten Bits
SPI_MasterTransmit(adresse & 0xFF);

CS_l;
}



void ERAL_93AA86C	(void)		//Erase/Write Enable (EWEN)			14 clock-cycles
{
CS_h;	

SPI_MasterTransmit(ERAL_high);
SPI_MasterTransmit(ERAL_low);

CS_l;
}

