// Ansteuerung eines HD44780 kompatiblen LCD im 4-Bit-Interfacemodus
// http://www.mikrocontroller.net/articles/AVR-GCC-Tutorial/LCD-Ansteuerung
//
 
#ifndef LCD_ROUTINES_H
#define LCD_ROUTINES_H
 
////////////////////////////////////////////////////////////////////////////////
// Hier die verwendete Taktfrequenz in Hz eintragen, wichtig!
 
#ifndef F_CPU
#define F_CPU 8000000
#endif
 
////////////////////////////////////////////////////////////////////////////////
// Pinbelegung fr das LCD, an verwendete Pins anpassen
// Alle LCD Pins mssen an einem Port angeschlossen sein und die 4
// Datenleitungen mssen auf aufeinanderfolgenden Pins liegen
/* 
Belegung in der neuen Schaltung: 
direkt angeschlossenes LCD mit Beleuchtung, 16 poliger Stecker
E		PC1, Pin22 
RS		PW0, Pin31

D4		PA4, Pin35
D5		PA5, Pin34
D6		PA6, Pin33
D7		PA7, Pin32
*/ 


//  LCD DB4-DB7 <-->  PORTD Bit PD4-PD7
#define LCD_DATA_PORT PORTA
#define LCD_DATA_DDR  DDRA
#define LCD_DB        PA4

/*
//  LCD DB4-DB5 <-->  PORTB Bit PB6-PB7
#define LCD_DATA_PORT1 PORTB
#define LCD_DATA_DDR1  DDRB
//#define LCD_DB        PB4		//<<-- hier noch korigieren oder prfen

//  LCD DB6-DB7 <-->  PORTD Bit PD5-PD6
#define LCD_DATA_PORT2 PORTD
#define LCD_DATA_DDR2  DDRD
//#define LCD_DB        PD4		//<<-- hier noch korigieren oder prfen
*/

#define LCD_STEUER_PORT PORTE
#define LCD_STEUER_DDR DDRE
//  LCD RS      <-->  PORTB Bit PB0     (RS: 1=Data, 0=Command)
#define LCD_RS        PE0


//Anpassung an die LCD-Anschlsse erfolgt in "lcd_out( uint8_t data )"


//das RW deaktivieren
//RW soll vorerst immer low = schreiben sein. Dieser Zustand mu vor jeder Ausgabe auf das LCD neu hergestellt werden.
//  LCD RW      <-->  PORTB Bit PB7     (RW: 1=lesen, 0=schreiben)
//#define LCD_RW        PB7



#define LCD_ENABLE_PORT PORTC
#define LCD_ENABLE_DDR DDRC 
//  LCD EN      <-->  PORTD Bit PD4     (EN: 1-Impuls fr Daten)
#define LCD_EN        PC1


////////////////////////////////////////////////////////////////////////////////
// LCD Ausfhrungszeiten (MS=Millisekunden, US=Mikrosekunden)
/* 
originale Werte
#define LCD_BOOTUP_MS           15
#define LCD_ENABLE_US           20
#define LCD_WRITEDATA_US        46
#define LCD_COMMAND_US          42
 
#define LCD_SOFT_RESET_MS1      5
#define LCD_SOFT_RESET_MS2      1
#define LCD_SOFT_RESET_MS3      1
#define LCD_SET_4BITMODE_MS     5
 
#define LCD_CLEAR_DISPLAY_MS    2
#define LCD_CURSOR_HOME_MS      2
*/
 
//verlngerte Werte
#define LCD_BOOTUP_MS           50
#define LCD_ENABLE_US           40
#define LCD_WRITEDATA_US        70
#define LCD_COMMAND_US          70
 
#define LCD_SOFT_RESET_MS1      10
#define LCD_SOFT_RESET_MS2      2
#define LCD_SOFT_RESET_MS3      2
#define LCD_SET_4BITMODE_MS     10
 
#define LCD_CLEAR_DISPLAY_MS    2
#define LCD_CURSOR_HOME_MS      2

 
////////////////////////////////////////////////////////////////////////////////
// Zeilendefinitionen des verwendeten LCD
// Die Eintrge hier sollten fr ein LCD mit einer Zeilenlnge von 16 Zeichen passen
// Bei anderen Zeilenlngen mssen diese Eintrge angepasst werden
 
#define LCD_DDADR_LINE1         0x00
#define LCD_DDADR_LINE2         0x40
#define LCD_DDADR_LINE3         0x10
#define LCD_DDADR_LINE4         0x50
 
////////////////////////////////////////////////////////////////////////////////
// Initialisierung: muss ganz am Anfang des Programms aufgerufen werden.
void lcd_init( void );
 
////////////////////////////////////////////////////////////////////////////////
// LCD lschen
void lcd_clear( void );
 
////////////////////////////////////////////////////////////////////////////////
// Cursor in die 1. Zeile, 0-te Spalte
void lcd_home( void );
 
////////////////////////////////////////////////////////////////////////////////
// Cursor an eine beliebige Position 
void lcd_setcursor( uint8_t spalte, uint8_t zeile );
 
////////////////////////////////////////////////////////////////////////////////
// Ausgabe eines einzelnen Zeichens an der aktuellen Cursorposition 
void lcd_data( uint8_t data );
 
////////////////////////////////////////////////////////////////////////////////
// Ausgabe eines Strings an der aktuellen Cursorposition 
void lcd_string( const char *data );
 
////////////////////////////////////////////////////////////////////////////////
// Definition eines benutzerdefinierten Sonderzeichens.
// data muss auf ein Array[5] mit den Spaltencodes des zu definierenden Zeichens
// zeigen
void lcd_generatechar( uint8_t code, const uint8_t *data );
 
////////////////////////////////////////////////////////////////////////////////
// Ausgabe eines Kommandos an das LCD.
void lcd_command( uint8_t data );
 
 
////////////////////////////////////////////////////////////////////////////////
// LCD Befehle und Argumente.
// Zur Verwendung in lcd_command
 
// Clear Display -------------- 0b00000001
#define LCD_CLEAR_DISPLAY       0x01
 
// Cursor Home ---------------- 0b0000001x
#define LCD_CURSOR_HOME         0x02
 
// Set Entry Mode ------------- 0b000001xx
#define LCD_SET_ENTRY           0x04
 
#define LCD_ENTRY_DECREASE      0x00
#define LCD_ENTRY_INCREASE      0x02
#define LCD_ENTRY_NOSHIFT       0x00
#define LCD_ENTRY_SHIFT         0x01
 
// Set Display ---------------- 0b00001xxx
#define LCD_SET_DISPLAY         0x08
 
#define LCD_DISPLAY_OFF         0x00
#define LCD_DISPLAY_ON          0x04
#define LCD_CURSOR_OFF          0x00
#define LCD_CURSOR_ON           0x02
#define LCD_BLINKING_OFF        0x00
#define LCD_BLINKING_ON         0x01
 
// Set Shift ------------------ 0b0001xxxx
#define LCD_SET_SHIFT           0x10
 
#define LCD_CURSOR_MOVE         0x00
#define LCD_DISPLAY_SHIFT       0x08
#define LCD_SHIFT_LEFT          0x00
#define LCD_SHIFT_RIGHT         0x04
 
// Set Function --------------- 0b001xxxxx
#define LCD_SET_FUNCTION        0x20
 
#define LCD_FUNCTION_4BIT       0x00
#define LCD_FUNCTION_8BIT       0x10
#define LCD_FUNCTION_1LINE      0x00
#define LCD_FUNCTION_2LINE      0x08
#define LCD_FUNCTION_5X7        0x00
#define LCD_FUNCTION_5X10       0x04
 
#define LCD_SOFT_RESET          0x30
 
// Set CG RAM Address --------- 0b01xxxxxx  (Character Generator RAM)
#define LCD_SET_CGADR           0x40
 
#define LCD_GC_CHAR0            0
#define LCD_GC_CHAR1            1
#define LCD_GC_CHAR2            2
#define LCD_GC_CHAR3            3
#define LCD_GC_CHAR4            4
#define LCD_GC_CHAR5            5
#define LCD_GC_CHAR6            6
#define LCD_GC_CHAR7            7
 
// Set DD RAM Address --------- 0b1xxxxxxx  (Display Data RAM)
#define LCD_SET_DDADR           0x80
 
#endif 
